@@ -1,4 +1,4 @@
-# Copyright 2014 PressLabs SRL
+# Copyright 2014-2016 Presslabs SRL
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,4 +14,4 @@
 
 
 class Node(object):
-    __slots__ = 'prev', 'next', 'data'
+    __slots__ = "prev", "next", "data"
