@@ -1,4 +1,4 @@
-# Copyright 2014 PressLabs SRL
+# Copyright 2014-2016 Presslabs SRL
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,8 +31,7 @@ class retry(object):
                 except:
                     time.sleep(self.each)
 
-                if (isinstance(self.times, int) and not
-                   isinstance(self.times, bool)):
+                if isinstance(self.times, int) and not isinstance(self.times, bool):
                     self.times -= 1
 
             return f(*args, **kwargs)
