@@ -1,4 +1,4 @@
-# Copyright 2014 PressLabs SRL
+# Copyright 2014-2016 Presslabs SRL
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,6 +15,8 @@
 
 from threading import Thread
 
+from six import iteritems
+
 from gitfs.log import log
 
 
@@ -22,7 +24,7 @@ class Peasant(Thread):
     def __init__(self, *args, **kwargs):
         super(Peasant, self).__init__()
 
-        for name, value in kwargs.iteritems():
+        for name, value in iteritems(kwargs):
             setattr(self, name, value)
 
     def run(self):
