@@ -1,4 +1,4 @@
-# Copyright 2014 PressLabs SRL
+# Copyright 2014-2016 Presslabs SRL
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,36 +11,57 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from Queue import Empty
+import random
+import time
 
 import pygit2
+from six.moves.queue import Empty
 
 from gitfs.worker.peasant import Peasant
 from gitfs.merges import AcceptMine
 
-from gitfs.events import (fetch, syncing, sync_done, writers, shutting_down,
-                          remote_operation, push_successful)
+from gitfs.events import (
+    fetch,
+    syncing,
+    sync_done,
+    writers,
+    shutting_down,
+    remote_operation,
+    push_successful,
+    idle,
+)
 from gitfs.log import log
 
 
 class SyncWorker(Peasant):
-    name = 'SyncWorker'
+    name = "SyncWorker"
 
-    def __init__(self, author_name, author_email, commiter_name,
-                 commiter_email, strategy=None, *args, **kwargs):
+    def __init__(
+        self,
+        author_name,
+        author_email,
+        commiter_name,
+        commiter_email,
+        strategy=None,
+        *args,
+        **kwargs
+    ):
         super(SyncWorker, self).__init__(*args, **kwargs)
 
         self.author = (author_name, author_email)
         self.commiter = (commiter_name, commiter_email)
 
-        strategy = strategy or AcceptMine(self.repository, author=self.author,
-                                          commiter=self.commiter,
-                                          repo_path=self.repo_path)
+        strategy = strategy or AcceptMine(
+            self.repository,
+            author=self.author,
+            commiter=self.commiter,
+            repo_path=self.repo_path,
+        )
         self.strategy = strategy
         self.commits = []
 
     def work(self):
+        idle_times = 0
         while True:
             if shutting_down.is_set():
                 log.info("Stop sync worker")
@@ -48,11 +69,19 @@ class SyncWorker(Peasant):
 
             try:
                 job = self.commit_queue.get(timeout=self.timeout, block=True)
-                if job['type'] == 'commit':
+                if job["type"] == "commit":
                     self.commits.append(job)
                 log.debug("Got a commit job")
+
+                idle_times = 0
+                idle.clear()
             except Empty:
                 log.debug("Nothing to do right now, going idle")
+
+                if idle_times > self.min_idle_times:
+                    idle.set()
+
+                idle_times += 1
                 self.on_idle()
 
     def on_idle(self):
@@ -77,8 +106,21 @@ class SyncWorker(Peasant):
                 log.info("Get some commits")
                 self.commit(self.commits)
                 self.commits = []
-            log.debug("Start syncing")
-            self.sync()
+
+            count = 0
+            log.debug("Start syncing, first attempt.")
+            while not self.sync() and count < 5:
+                fuzz = random.randint(0, 1000) / 1000
+                wait = 2 ** count + fuzz
+
+                log.debug("Failed to sync. Going to sleep for %d seconds", wait)
+                time.sleep(wait)
+
+                count += 1
+                log.debug("Retry-ing to sync with remote. Attempt #%d", count)
+
+            if count >= 5:
+                log.error("Didn't manage to sync, I need some help")
 
     def merge(self):
         log.debug("Start merging")
@@ -98,17 +140,22 @@ class SyncWorker(Peasant):
         if self.repository.behind:
             log.debug("I'm behind so I start merging")
             try:
+                log.debug("Start fetching")
+                self.repository.fetch(self.upstream, self.branch, self.credentials)
+                log.debug("Done fetching")
+                log.debug("Start merging")
                 self.merge()
+                log.debug("Merge done with success, ready to push")
                 need_to_push = True
             except:
                 log.exception("Merge failed")
-                return
+                return False
 
         if need_to_push:
             try:
                 with remote_operation:
                     log.debug("Start pushing")
-                    self.repository.push(self.upstream, self.branch)
+                    self.repository.push(self.upstream, self.branch, self.credentials)
                     self.repository.behind = False
                     log.info("Push done")
                 log.debug("Clear syncing")
@@ -117,36 +164,53 @@ class SyncWorker(Peasant):
                 sync_done.set()
                 log.debug("Set push_successful")
                 push_successful.set()
-            except:
+            except Exception as error:
                 push_successful.clear()
                 fetch.set()
-                log.exception("Push failed")
+                log.debug("Push failed because of %s", error)
+                return False
         else:
+            log.debug("Sync done, clearing")
             sync_done.set()
             syncing.clear()
 
+        return True
+
     def commit(self, jobs):
         if len(jobs) == 1:
-            message = jobs[0]['params']['message']
+            message = jobs[0]["params"]["message"]
         else:
             updates = set([])
+            number_of_removal = 0
+            number_of_additions = 0
             for job in jobs:
-                updates = updates | set(job['params']['add'])
-                updates = updates | set(job['params']['remove'])
-
-            message = "Update %s items" % len(updates)
+                removal_set = set(job["params"]["remove"])
+                addition_set = set(job["params"]["add"])
+                number_of_removal += len(removal_set)
+                number_of_additions += len(addition_set)
+                updates = updates | removal_set | addition_set
+            message = "Update {} items. ".format(len(updates))
+            if number_of_additions:
+                message += "Added {} items. ".format(number_of_additions)
+            if number_of_removal:
+                message += "Removed {} items. ".format(number_of_removal)
+            message = message.strip()
 
         old_head = self.repository.head.target
-        new_commit = self.repository.commit(message, self.author,
-                                            self.commiter)
+        new_commit = self.repository.commit(message, self.author, self.commiter)
 
         if new_commit:
-            log.debug("Commit %s with %s as author and %s as commiter",
-                      message, self.author, self.commiter)
+            log.debug(
+                "Commit %s with %s as author and %s as commiter",
+                message,
+                self.author,
+                self.commiter,
+            )
             self.repository.commits.update()
             log.debug("Update commits cache")
         else:
-            self.repository.create_reference("refs/heads/%s" % self.branch,
-                                             old_head, force=True)
+            self.repository.create_reference(
+                "refs/heads/%s" % self.branch, old_head, force=True
+            )
         self.repository.checkout_head(strategy=pygit2.GIT_CHECKOUT_FORCE)
         log.debug("Checkout to HEAD")
